@@ -3,6 +3,22 @@
 # NEED TO ADD A VALIDATOR
 setClass("dictionary", contains = "list")
 
+#' print a dictionary object
+#' 
+#' Print/show method for dictionary objects.
+#' @param object the dictionary to be printed
+#' @export
+setMethod("show", "dictionary", 
+          function(object) {
+              cat("Dictionary object with", length(object), "key entries.\n")
+              keys <- names(object)
+              lapply(seq_along(object), 
+                     function(i, object, keys)
+                         cat(" - ", keys[i], ": ", paste(object[[i]], collapse = ", "), "\n", sep = ""),
+                     object = object, keys = names(object))
+              
+              # print(setClass("list", object))
+          })
 
 #' create a dictionary
 #' 
@@ -186,14 +202,18 @@ readWStatDictNested <- function(path) {
 # LIWCdict <- readLIWCdict("~/Dropbox/QUANTESS/corpora/LIWC/LIWC2001_English.dic") }
 readLIWCdict <- function(path, maxcats=10, enc="") {
     # read in the dictionary as a (big, uneven) table
-    d <- utils::read.delim(path, header=FALSE, fileEncoding=enc,
-                    col.names=c("category", paste("catno", 1:maxcats, sep="")),
-                    stringsAsFactors=FALSE)
-    
+    d <- utils::read.table(path, header=FALSE, fileEncoding=enc,
+                           col.names=c("category", paste("catno", 1:maxcats)),
+                           fill=TRUE, stringsAsFactors=FALSE)
     # get the row number that signals the end of the category guide
     guideRowEnd <- max(which(d$category=="%"))
+    if(guideRowEnd < 1){
+        stop('Expected a guide (a category legend) delimted by percentage symbols at start of file, none found')
+    }
     # extract the category guide
     guide <- d[2:(guideRowEnd-1), 1:2]
+    colnames(guide) <- c('catNum', 'catName' )
+    guide$catNum <- as.numeric(guide$catNum)
     # initialize the dictionary as list of NAs
     dictionary <- list()
     length(dictionary) <- nrow(guide)
@@ -212,10 +232,11 @@ readLIWCdict <- function(path, maxcats=10, enc="") {
         terms[[i]] <- as.numeric(catlist[i, !is.na(catlist[i,])])
     }
 
-    # now fill the dictionary with the terms
-    for (t in 1:length(terms)) {
-        for (c in terms[[t]]) {
-            dictionary[[c]] <- append(dictionary[[c]], names(terms[t]))
+    for(ind in 1:length(terms)){
+        for(num in as.numeric(terms[[ind]])){
+            thisCat <- guide$catName[which(guide$catNum==num)]
+            thisTerm <- names(terms[ind])
+            dictionary[[thisCat]] <- append(dictionary[[thisCat]], thisTerm)
         }
     }
     return(dictionary)
@@ -245,3 +266,108 @@ flatten.dictionary <- function(elms, parent = '', dict = list()) {
     return(dict)
 }
 
+#' apply a dictionary or thesarus to an object
+#' 
+#' Convert features into equivalence classes defined by values of a dictionary
+#' object.  
+#' @note Selecting only features defined in a "dictionary" is traditionally
+#' known in text analysis as a dictionary method, even though technically this is more like a thesarus.
+#' If a more truly thesaurus-like application is desired, set \code{keeponly = FALSE} to convert features 
+#' defined as values in a dictionary into their keys, while keeping all other features.
+#' @return an object of the type passed with the value-matching features replaced by dictionary keys
+#' @param x object to which dictionary or thesaurus will be supplied
+#' @param dictionary the \link{dictionary}-class object that will be applied to \code{x}
+#' @export
+applyDictionary <- function(x, dictionary, ...) {
+    UseMethod("applyDictionary")
+}
+
+#' @rdname applyDictionary
+#' @param exclusive if \code{TRUE}, remove all features not in dictionary, 
+#'   otherwise, replace values in dictionary keys with keys while leaving other 
+#'   features unaffected
+#' @param valuetype how to interpret dictionary values: \code{"glob"} for 
+#'   "glob"-style wildcard expressions (the format used in Wordstat and LIWC
+#'   formatted dictionary values); \code{"regex"} for regular expressions; or
+#'   \code{"fixed"} for exact matching (entire words, for instance)
+#' @param case_insensitive ignore the case of dictionary values if \code{TRUE}
+#' @param capkeys if \code{TRUE}, convert dictionary or thesaurus keys to 
+#'   uppercase to distinguish them from other features
+#' @param verbose print status messages if \code{TRUE}
+#' @param ... not used
+#' @export
+#' @examples
+#' myDict <- dictionary(list(christmas = c("Christmas", "Santa", "holiday"),
+#'                           opposition = c("Opposition", "reject", "notincorpus"),
+#'                           taxglob = "tax*",
+#'                           taxregex = "tax.+$",
+#'                           country = c("United_States", "Sweden")))
+#' myDfm <- dfm(c("My Christmas was ruined by your opposition tax plan.", 
+#'                "Does the United_States or Sweden have more progressive taxation?"),
+#'              ignoredFeatures = stopwords("english"), verbose = FALSE)
+#' myDfm
+#' 
+#' # glob format
+#' applyDictionary(myDfm, myDict, valuetype = "glob")
+#' applyDictionary(myDfm, myDict, valuetype = "glob", case_insensitive = FALSE)
+#'
+#' # regex v. glob format: note that "united_states" is a regex match for "tax*"
+#' applyDictionary(myDfm, myDict, valuetype = "glob")
+#' applyDictionary(myDfm, myDict, valuetype = "regex", case_insensitive = TRUE)
+#' 
+#' # fixed format: no pattern matching
+#' applyDictionary(myDfm, myDict, valuetype = "fixed")
+#' applyDictionary(myDfm, myDict, valuetype = "fixed", case_insensitive = FALSE)
+applyDictionary.dfm <- function(x, dictionary, exclusive = TRUE, valuetype = c("glob", "regex", "fixed"), 
+                                case_insensitive = TRUE,
+                                capkeys = !exclusive,
+                                verbose = TRUE, ...) {
+    valuetype <- match.arg(valuetype)
+    dictionary <- flatten.dictionary(dictionary)
+    
+    if (verbose) cat("applying a dictionary consisting of ", length(dictionary), " key", 
+                     ifelse(length(dictionary) > 1, "s", ""), "\n", sep="")
+    
+    # convert wildcards to regular expressions (if needed)
+    if (valuetype == "glob") {
+        dictionary <- lapply(dictionary, utils::glob2rx)
+    } # else if (valuetype == "fixed")
+    # dictionary <- lapply(dictionary, function(x) paste0("^", x, "$"))
+    
+    newDocIndex <- rep(1:nrow(x), length(dictionary))
+    newFeatures <- names(dictionary)
+    uniqueFeatures <- features(x)
+    newFeatureIndexList <- lapply(dictionary, function(x) {
+        # ind <- grep(paste(x, collapse = "|"), uniqueFeatures, ignore.case = case_insensitive)
+        if (valuetype == "fixed") {
+            if (case_insensitive)  
+                ind <- which(toLower(uniqueFeatures) %in% (toLower(x)))
+            else ind <- which(uniqueFeatures %in% x)
+        }
+        else ind <- which(stringi::stri_detect_regex(uniqueFeatures, paste(x, collapse = "|"), case_insensitive = case_insensitive))
+        if (length(ind) == 0)
+            return(NULL)
+        else 
+            return(ind)
+    })
+    if (capkeys) newFeatures <- stringi::stri_trans_toupper(newFeatures)
+    newFeatureCountsList <- lapply(newFeatureIndexList,
+                                   function(i) {
+                                       if (!is.null(i)) 
+                                           rowSums(x[, i])
+                                       else 
+                                           rep(0, nrow(x))
+                                   })
+    dfmresult2 <- sparseMatrix(i = newDocIndex,
+                               j = rep(1:length(dictionary), each = ndoc(x)),
+                               x = unlist(newFeatureCountsList),
+                               dimnames=list(docs = docnames(x), 
+                                             features = newFeatures))
+    if (!exclusive) {
+        keyIndex <- unlist(newFeatureIndexList, use.names = FALSE)
+        dfmresult2 <- cbind(x[, -keyIndex], dfmresult2)
+    }
+    
+    new("dfmSparse", dfmresult2)
+}
+
